#include <stdio.h>
#include <math.h>
#include <string.h>
#include <sys/types.h>
#include <sys/stat.h>
#include "defs.h"


void exit();

struct pxm_img
{
  int			height;
  int			width;
  int			max_gray;
  char			pxm_type;  /* 'b' = bitmap; 'g' = graymap;  */
				   /* 'p' = pixel map               */
  unsigned char		**mono;
  unsigned char		***color;
};



static int write_pxm();
static int read_pxm_header();




void writearray(array,length,name)	
/* writes out a file of character data */ 
unsigned char	*array;
char	*name;
int	length;
{
	int	creat(),fp;

	if((fp = creat(name,0644))==-1) {
		fprintf(stderr,"writearray: can't create file %s\n",name); 
		exit(1); }

	write(fp,(char *)array,(unsigned int)length);
	close(fp);
}

void readarray(array,length,name)	
/* reads a file of character data */
unsigned char	*array;
char	*name;
int	length;
{
	int	open(),fp;
	long	i,n;

	if((fp = open(name,0))==-1) {
		fprintf(stderr,"readarray: can't open file %s\n",name); exit(1); }
	i = 0;
	while((n=read(fp,(char *)array+i,8192))>0 ) i += n;
	if(i!=length) { 
		fprintf(stderr,"readarray: incorrect file size, %s\n",name); 
		}
	close(fp);
}

void write_img(image,wd,ht,name)	
/* writes out a 2D file of class image data */ 
IMAGE_CLASS **image; 	/* image with dimentions image[ht][wd] */
int	wd,ht;
char	*name;		/* file name */
{
	FILE   *fp;
        struct pxm_img img;
        int i,j;

 	fp = fopen(name,"w");
 	if (fp == NULL) {
	  fprintf(stderr,"write_img: can't open file %s\n",name);
   	  exit (-1);
 	}

	img.pxm_type = 'g';
	img.width = wd;
	img.height = ht;
	img.max_gray = 255;

        /* Convert segmentation image to unsigned char */
	img.mono = (unsigned char **)
                   get_img(img.width,img.height,sizeof(unsigned char));
        for(i=0; i<img.height; i++)
        for(j=0; j<img.width; j++) {
	  img.mono[i][j] = (unsigned char)image[i][j];
        }

	if( write_pxm(fp, &img) ) {
	  fprintf(stderr,"\nwrite_img: error writing out file %s\n",name);
          exit (-1);
        }

        free_img((char **)img.mono);

	fclose(fp);
}

void read_img(image,wd,ht,name)	
/* reads a file 2D of character data */
unsigned char	**image;	/* image with dimentions image[ht][wd] */
int	wd,ht;
char	*name;		/* file name */
{
	FILE   *fp;
        struct pxm_img img;
	long	i;

 	fp = fopen(name,"r");
 	if (fp == NULL) {
	  fprintf(stderr,"read_img: can't open file %s\n",name);
   	  exit (-1);
 	}

	if( read_pxm_header (fp, &img) ) {
	  fprintf (stderr, "read_img: error reading header for input file %s\n",name);
	  exit (-1);
	}

	if(img.pxm_type != 'g') {
          fprintf(stderr,"read_img: input file %s must be pgm\n",name); 
	  exit(1); 
	}
	if((img.width != wd)||(img.height!=ht)) {
          fprintf(stderr,"read_img: input file %s has inconsistant width or height\n",name); 
	  exit(1); 
	}

	for(i=0; i<ht; i++) {
	  if(wd!=fread((unsigned char *)image[i],sizeof(char),wd,fp)) {
	    fprintf(stderr,"read_img: input file %s corrupted\n",name); 
	    exit(1);
	  }
	}

	fclose(fp);
}




/*************************************************************************/
/* write_pxm()                                                           */
/*  routine that takes in a binary, grayscale, or color image		 */
/*  and writes it out in pbm, pgm, or ppm format. It returns a           */
/*  value 1 if it was not able to write out the data. Otherwise, it      */
/*  returns 0. 							         */
/*************************************************************************/

static int write_pxm (fout, img)
FILE *fout;
struct pxm_img *img;
{
  int			i, j;
  unsigned char		junk = 0;
  int			status;
  int			magic_num;
  unsigned char		*temp;
  int			k, m;
  int			full = 0;
  int			count;
  int			bufsize;
  int			imgsize;
  static unsigned char	mask[8] = {128, 64, 32, 16, 8, 4, 2, 1};
  
  status = 0;
  magic_num = 5;

  if ((img->pxm_type != 'p') && (img->pxm_type != 'g') &&
    (img->pxm_type != 'b'))
      return (1);

  if ((img->height <= 0) || (img->width <= 0))
    return (1);

  if (img->pxm_type == 'p')
    magic_num = 6;
  else if (img->pxm_type == 'g')
    magic_num = 5;
  else
    magic_num = 4;

  img->max_gray = 255;

/* write header info to output file */

  if (magic_num > 4) /* print max_gray if not a bitmap */ {
    if (fprintf (fout, "P%d\n%d\n%d\n%d\n", magic_num, img->width, 
      img->height, img->max_gray) == EOF) 
        status = 1;
  }
  else /* bitmap */ {
    if (fprintf (fout, "P%d\n%d\n%d\n", magic_num, img->width, 
      img->height) == EOF) 
        status = 1;
  }

/* write image to output file */

  if (magic_num < 6) /* not color */ {
    if (magic_num == 5) /* grayscale */ {
      for (i = 0; i < img->height; i++) 
        if (img->width != fwrite ((char *) img->mono[i], 
	  sizeof (unsigned char), img->width, fout)) 
            status = 1;
    }
    else /* bitmap */ {
      bufsize = img->width / 8;
      if (img->width % 8) 
        bufsize++;
      temp = (unsigned char *) malloc (bufsize * sizeof (unsigned char));
      for (i = 0; i < img->height; i++) {
	full = 0;
	j = 0;
        for (k = 0; k < bufsize; k++) {
          temp[k] = 0;
	  for (m = 0; m < 8; m++) {
	    if (!full) 
	      temp[k] |= ((img->mono[i][j]) ? 0 : mask[m]);
	    if (++j >= img->width)
              full = 1;
	  }
	}
        if (bufsize != fwrite ((unsigned char *) temp, sizeof (unsigned
          char), bufsize, fout)) {
            fprintf (stderr, "Failed to write %d bytes to output file.\n",
              bufsize);
            return (1);
        }
      }
      free (temp);
    }
  }
  else /* color */ {
    temp = (unsigned char *) malloc (sizeof (unsigned char) * 3 * 
      img->width);
    for (i = 0; i < img->height; i++) {
      m = 0;
      for (j = 0; j < img->width; j++)
        for (k = 0; k < 3; k++)
	  temp[m++] = img->color[k][i][j];

      if (3 * img->width != fwrite ((char *) temp, 
	sizeof (unsigned char), 3 * img->width, fout)) 
	  status = 1;
    }
    free (temp);
  }
  return(status);
}



static int read_pxm_header (fin, img)
FILE *fin; 
struct pxm_img *img;
{
  char			str[75];
  char			*str2;
  char			magic_char;
  char			magic_num;
  int			num_items = 0;
  int			max_items;
  char			**items;

  items = (char **) malloc (sizeof (char *) * 4);
  if (fgets (str, 3, fin) == NULL) /* failed read or EOF */
    return (1);
  else if ((str[0] != 'P') || ((str[1] != '4') && (str[1] != '5')
    && (str[1] != '6')))  /* invalid "magic number" */
    return (1);

  if (str[1] == '4')
    img->pxm_type = 'b';
  else if (str[1] == '5')
    img->pxm_type = 'g';
  else
    img->pxm_type = 'p';

  max_items = (img->pxm_type == 'b') ? 2 : 3;

  while (num_items < max_items) {
    if (fgets (str, 72, fin) == NULL) /* failed read or EOF */
      return (1);
    if (strlen (str) > (size_t) 70) /* line too long */
      return (1);
    if ((str2 = strchr (str, '#')) != NULL) /* strip comments */
      str2[0] = '\0';
    str2 = strtok (str, " \t\n\r\f\v");
    while (str2 != NULL) {
      num_items++;
      if (num_items > max_items)
        return (1);
      items[num_items] = (char *) malloc (sizeof (char) * (strlen (str2)
        +1));
      strcpy (items[num_items], str2);
      str2 = strtok (NULL, " \t\n\r\f\v");
    }
  }

  img->width = atoi (items[1]);
  if (img->width <= 0)
    return (1);
  img->height = atoi (items[2]);
  if (img->height <= 0)
    return (1);
  if (img->pxm_type != 'b') {
    img->max_gray = atoi (items[3]);
    if ((img->max_gray < 0) || (img->max_gray > 255))
      return (1);
  }
  free (items);
  return (0);
}

