#include <stdio.h>
#include "sig.h"

#define eq(a,b) strcmp(a,b)==0

I_ReadSigSet (fd, S)
    FILE *fd;
    struct SigSet *S;
{
    char tag[256];

    I_InitSigSet (S);

    while (gettag(fd, tag))
    {
	if (eq (tag, "title:"))
	    get_title(fd, S);
	if (eq (tag, "nbands:"))
	    get_nbands(fd, S);
	if (eq (tag, "class:"))
	    get_class (fd, S);
    }
}

static gettag(fd, tag)
    FILE *fd;
    char *tag;
{
    if(fscanf(fd, "%s", tag) != 1) return 0;
    G_strip (tag);
    return 1;
}

static
get_nbands(fd, S)
    FILE *fd;
    struct SigSet *S;
{
    fscanf (fd, "%d", &S->nbands);
}

static
get_title(fd, S)
    FILE *fd;
    struct SigSet *S;
{
    char title[1024];

    *title = 0;
    fscanf (fd, "%[\n]", title);
    I_SetSigTitle(S, title);
}

static
get_class(fd, S)
    FILE *fd;
    struct SigSet *S;
{
    char tag[1024];
    struct ClassSig *C;

    printf ("get_class\n");
    C = I_NewClassSig(S);
    while (gettag(fd, tag))
    {
	if (eq(tag, "endclass:"))
	    break;
	if (eq(tag, "classnum:"))
	    get_classnum(fd, C);
	if (eq(tag, "classtype:"))
	    get_classtype(fd, C);
	if (eq(tag, "classtitle:"))
	    get_classtitle(fd, C);
	if (eq(tag, "subclass:"))
	    get_subclass(fd, S, C);
    }
}

static
get_classnum(fd, C)
    FILE *fd;
    struct ClassSig *C;
{
    fscanf (fd, "%d", &C->classnum);
}

static
get_classtype(fd, C)
    FILE *fd;
    struct ClassSig *C;
{
    fscanf (fd, "%d", &C->type);
}

static
get_classtitle(fd, C)
    FILE *fd;
    struct ClassSig *C;
{
    char title[1024];

    *title = 0;
    fscanf (fd, "%[\n]", title);
    I_SetClassTitle (C, title);
}

static
get_subclass (fd, S, C)
    FILE *fd;
    struct SigSet *S;
    struct ClassSig *C;
{
    struct SubSig *Sp;
    char tag[1024];

    Sp = I_NewSubSig(S,C);

    while (gettag(fd, tag))
    {
	if (eq(tag, "endsubclass:"))
	    break;
	if (eq(tag, "pi:"))
	    get_subclass_pi(fd, Sp);
	if (eq(tag, "means:"))
	    get_subclass_means (fd, Sp, S->nbands);
	if (eq(tag, "covar:"))
	    get_subclass_covar (fd, Sp, S->nbands);
    }
}

static
get_subclass_pi(fd, Sp)
    FILE *fd;
    struct SubSig *Sp;
{
    fscanf (fd, "%lf", &Sp->pi);
}

static
get_subclass_means (fd, Sp, nbands)
    FILE *fd;
    struct SubSig *Sp;
    int nbands;
{
    int i;

    for (i = 0; i < nbands; i++)
	fscanf (fd, "%lf", &Sp->means[i]);
}

static
get_subclass_covar (fd, Sp, nbands)
    FILE *fd;
    struct SubSig *Sp;
    int nbands;
{
    int i,j;

    for (i = 0; i < nbands; i++)
	for (j = 0; j < nbands; j++)
	    fscanf (fd, "%lf", &Sp->R[i][j]);
}
